package edu.sfedu.brsback.dto.module;

import edu.sfedu.brsback.entities.Module;
import edu.sfedu.brsback.entities.User;
import edu.sfedu.brsback.enums.ModuleType;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

import java.util.List;

@AllArgsConstructor
@NoArgsConstructor
@Getter
@Setter
public class ModuleResponse {
    private Long id;
    private String title;
    private List<String> teachers;
    private ModuleType moduleType;
    private Integer maxScore;
    private Integer currentScore = 0;
    private Integer percent = 0;
    private List<ModuleResponse> children;

    public ModuleResponse(Module module) {
        id = module.getId();
        title = module.getTitle();
        moduleType = module.getModuleType();
        maxScore = module.getMaxScore();
        teachers = module.getTeachers().stream().map(User::getFio).toList();
        children = module.getChildren().stream().map(ModuleResponse::new).toList();
    }

    public ModuleResponse(Module module, Long recordBookId) {
        id = module.getId();
        title = module.getTitle();
        moduleType = module.getModuleType();
        maxScore = module.getMaxScore();
        teachers = module.getTeachers().stream().map(User::getFio).toList();
        currentScore = module.getStudentScoreSum(recordBookId);
        percent = currentScore >= maxScore ?
                100 :
                (int) ((double) currentScore / maxScore * 100);
        children = module.getChildren().stream()
                .map((Module childModule) -> new ModuleResponse(childModule, recordBookId))
                .toList();
    }
}
