package edu.sfedu.brsback.entities;

import edu.sfedu.brsback.enums.ModuleType;
import jakarta.annotation.Nullable;
import jakarta.persistence.*;
import lombok.*;
import org.hibernate.proxy.HibernateProxy;

import java.util.*;

@Getter
@Setter
@Entity
@Builder
@Table(name = "module")
@NoArgsConstructor
@AllArgsConstructor
public class Module extends BaseEntity {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id", nullable = false)
    private Long id;

    @Column(name = "title", length = 120)
    private String title;

    @ManyToOne(cascade = CascadeType.REMOVE)
    @JoinColumn(name = "parent_id")
    @Nullable
    private Module parent;

    @OneToMany(mappedBy = "parent", orphanRemoval = true)
    @OrderBy
    private List<Module> children = new ArrayList<>();

    @ManyToOne(cascade = {CascadeType.DETACH})
    @JoinColumn(name = "education_period_id")
    private EducationPeriod educationPeriod;

    @Enumerated
    @Column(name = "module_type")
    private ModuleType moduleType;

    @Column(name = "max_score")
    private Integer maxScore;

    @ManyToMany(cascade = CascadeType.PERSIST)
    @JoinTable(name = "module_teacher",
            joinColumns = @JoinColumn(name = "module_id"),
            inverseJoinColumns = @JoinColumn(name = "teacher_user_id"))
    private Set<User> teachers = new LinkedHashSet<>();

    @OneToMany(mappedBy = "module", orphanRemoval = true)
    private Set<Score> scores = new LinkedHashSet<>();

    public Integer getStudentScoreSum(Long recordBookId){
        if (getChildren().isEmpty()){
            return scores.stream().filter((Score s) -> Objects.equals(s.getRecordBook().getId(), recordBookId))
                    .mapToInt(Score::getScore)
                    .sum();
        }
        Integer childrenSum = 0;
        for (Module childModule : getChildren()){
            childrenSum += childModule.getStudentScoreSum(recordBookId);
        }
        return childrenSum;
    }

    public Integer getStudentScoreSum(RecordBook recordBook){
        if (getChildren().isEmpty()){
            return scores.stream().filter((Score s) -> s.getRecordBook() == recordBook)
                    .mapToInt(Score::getScore)
                    .sum();
        }
        Integer childrenSum = 0;
        for (Module childModule : getChildren()){
            childrenSum += childModule.getStudentScoreSum(recordBook);
        }
        return childrenSum;
    }

    @Override
    public final boolean equals(Object o) {
        if (this == o) return true;
        if (o == null) return false;
        Class<?> oEffectiveClass = o instanceof HibernateProxy ? ((HibernateProxy) o).getHibernateLazyInitializer().getPersistentClass() : o.getClass();
        Class<?> thisEffectiveClass = this instanceof HibernateProxy ? ((HibernateProxy) this).getHibernateLazyInitializer().getPersistentClass() : this.getClass();
        if (thisEffectiveClass != oEffectiveClass) return false;
        Module module = (Module) o;
        return getId() != null && Objects.equals(getId(), module.getId());
    }

    @Override
    public final int hashCode() {
        return this instanceof HibernateProxy ? ((HibernateProxy) this).getHibernateLazyInitializer().getPersistentClass().hashCode() : getClass().hashCode();
    }
}