package edu.sfedu.brsback.entities;

import edu.sfedu.brsback.dto.ModuleRightDto;
import jakarta.persistence.*;
import lombok.*;

import java.util.UUID;

@Builder
@Getter
@Setter
@Entity
@Table(name = "module_right")
@NoArgsConstructor
@AllArgsConstructor
public class ModuleRight extends BaseEntity {
    @Id
    @GeneratedValue(strategy = GenerationType.UUID)
    @Column(name = "id", nullable = false)
    private UUID id;

    @ManyToOne(cascade = CascadeType.REMOVE)
    @JoinColumn(name = "module_id", nullable = false)
    private Module module;

    @ManyToOne(cascade = CascadeType.REMOVE)
    @JoinColumn(name = "user_id", nullable = false)
    private User user;

    @ManyToOne(cascade = CascadeType.REMOVE)
    @JoinColumn(name = "record_book_id")
    private RecordBook recordBook;

    @Column(name = "r_read")
    private Boolean rRead;

    @Column(name = "r_create")
    private Boolean rCreate;

    @Column(name = "r_edit")
    private Boolean rEdit;

    @Column(name = "r_delete")
    private Boolean rDelete;

    @Column(name = "r_score")
    private Boolean rScore;

    public ModuleRight(ModuleRightDto moduleRightDTO, User user, Module module) {
        rRead = moduleRightDTO.isRRead();
        rEdit = moduleRightDTO.isREdit();
        rDelete = moduleRightDTO.isRDelete();
        rCreate = moduleRightDTO.isRCreate();
        rScore = moduleRightDTO.isRScore();
        this.user = user;
        this.module = module;
    }

    public ModuleRight(ModuleRightDto moduleRightDTO, RecordBook recordBook, Module module) {
        rRead = moduleRightDTO.isRRead();
        rEdit = moduleRightDTO.isREdit();
        rDelete = moduleRightDTO.isRDelete();
        rCreate = moduleRightDTO.isRCreate();
        rScore = moduleRightDTO.isRScore();
        this.user = recordBook.getUser();
        this.module = module;
        this.recordBook = recordBook;
    }

    public void setRightFromDTO(ModuleRightDto moduleRightDTO){
        rRead = moduleRightDTO.isRRead();
        rEdit = moduleRightDTO.isREdit();
        rDelete = moduleRightDTO.isRDelete();
        rCreate = moduleRightDTO.isRCreate();
        rScore = moduleRightDTO.isRScore();
    }
}