package edu.sfedu.brsback.services.implementations;

import edu.sfedu.brsback.builders.ModuleRightBuilder;
import edu.sfedu.brsback.dto.ModuleRightDto;
import edu.sfedu.brsback.entities.Module;
import edu.sfedu.brsback.entities.ModuleRight;
import edu.sfedu.brsback.entities.RecordBook;
import edu.sfedu.brsback.entities.User;
import edu.sfedu.brsback.repos.ModuleRightRepository;
import edu.sfedu.brsback.services.ModuleRightService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.Optional;
import java.util.Set;

@Service
@RequiredArgsConstructor
public class ModuleRightServiceImpl implements ModuleRightService {
    private final ModuleRightRepository rightRepository;

    @Override
    public ModuleRightDto getRightOnModule(Module module, User user) {
        ModuleRightDto moduleRightDTO;
        switch (user.getUserRole()) {
            case MANAGER, TEACHER -> moduleRightDTO = rightRepository
                    .findDTOByModuleAndUser(module.getId(), user.getId())
                    .orElse(new ModuleRightDto(false, false, false, false, false));

            case ADMIN -> moduleRightDTO = new ModuleRightDto(true, true, true, true, true);

            case STUDENT -> {
                // Студент имеет права на модуль, если хоть одна его зачётка к нему прикреплена
                ModuleRightBuilder moduleRightBuilder = new ModuleRightBuilder();
                Set<RecordBook> recordBooks = user.getRecordBooks();
                for (RecordBook recordBook : recordBooks) {
                    ModuleRightDto rightOnRecordBook = getRightOnModule(module, recordBook);

                    moduleRightBuilder.setRCreate(rightOnRecordBook.isRCreate() || moduleRightBuilder.isRCreate());
                    moduleRightBuilder.setREdit(rightOnRecordBook.isREdit() || moduleRightBuilder.isREdit());
                    moduleRightBuilder.setRDelete(rightOnRecordBook.isRDelete() || moduleRightBuilder.isRDelete());
                    moduleRightBuilder.setRRead(rightOnRecordBook.isRRead() || moduleRightBuilder.isRRead());
                    moduleRightBuilder.setRScore(rightOnRecordBook.isRScore() || moduleRightBuilder.isRScore());
                }

                moduleRightDTO = moduleRightBuilder.toDTO();
            }

            default -> moduleRightDTO = new ModuleRightDto(false, false, false, false, false);
        }

        return moduleRightDTO;
    }

    @Override
    public ModuleRightDto getRightOnModule(Module module, RecordBook recordBook) {
        return rightRepository
                .findDTOByModuleAndRecordBook(module.getId(), recordBook.getId())
                .orElse(new ModuleRightDto(false, false, false, false, false));
    }

    @Override
    public boolean setRightOnModule(Module module, User user, ModuleRightDto rightDTO) {
        try {
            Optional<ModuleRight> moduleRightOpt = rightRepository
                    .findByModuleAndUser(module.getId(), user.getId());
            ModuleRight moduleRight;

            if (moduleRightOpt.isPresent()) {
                moduleRight = moduleRightOpt.get();
                moduleRight.setRightFromDTO(rightDTO);
            } else {
                moduleRight = new ModuleRight(rightDTO, user, module);
            }

            rightRepository.save(moduleRight);
        } catch (RuntimeException exception) {
            return false;
        }
        return true;
    }

    @Override
    public boolean setRightOnModule(Module module, RecordBook recordBook, ModuleRightDto rightDTO) {
        try {
            Optional<ModuleRight> moduleRightOpt = rightRepository
                    .findByModuleAndRecordBook(module.getId(), recordBook.getId());
            ModuleRight moduleRight;

            if (moduleRightOpt.isPresent()) {
                moduleRight = moduleRightOpt.get();
                moduleRight.setRightFromDTO(rightDTO);
            } else {
                moduleRight = new ModuleRight(rightDTO, recordBook, module);
            }

            rightRepository.save(moduleRight);
        } catch (RuntimeException exception) {
            return false;
        }
        return true;
    }
}
