package edu.sfedu.brsback.services.implementations;

import edu.sfedu.brsback.dto.auth.AuthCredentials;
import edu.sfedu.brsback.entities.User;
import edu.sfedu.brsback.enums.UserRole;
import edu.sfedu.brsback.exceptions.ModelNotFoundException;
import edu.sfedu.brsback.repos.UserRepository;
import edu.sfedu.brsback.services.SecurityService;
import edu.sfedu.brsback.services.auth.JwtService;
import edu.sfedu.brsback.services.auth.UserService;
import lombok.RequiredArgsConstructor;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

@Service
@RequiredArgsConstructor
public class SecurityServiceImpl implements SecurityService {
    private final UserService userService;
    private final JwtService jwtService;
    private final AuthenticationManager authenticationManager;

    @Override
    public String login(AuthCredentials creds) {
        authenticationManager.authenticate(new UsernamePasswordAuthenticationToken(
                creds.login(),
                creds.password()
        ));

        UserDetails user = userService
                .userDetailsService()
                .loadUserByUsername(creds.login());

        return jwtService.generateToken(user);
    }

    @Override
    public User getAuthUser() {
        return userService.getCurrentUser();
    }

    @Override
    public Long getAuthUserId() {
        return getAuthUser().getId();
    }

    @Override
    public boolean logout() {
        SecurityContextHolder.getContext().setAuthentication(null);
        return true;
    }
}
