package edu.sfedu.brsback.services.implementations;

import edu.sfedu.brsback.dto.visit.VisitSetDto;
import edu.sfedu.brsback.entities.Module;
import edu.sfedu.brsback.entities.RecordBook;
import edu.sfedu.brsback.entities.Visit;
import edu.sfedu.brsback.events.eventlog.VisitCreatedPublisher;
import edu.sfedu.brsback.exceptions.AccessDeniedException;
import edu.sfedu.brsback.exceptions.ModelNotFoundException;
import edu.sfedu.brsback.repos.RecordBookRepository;
import edu.sfedu.brsback.repos.VisitRepository;
import edu.sfedu.brsback.services.ModuleRightService;
import edu.sfedu.brsback.services.ModuleService;
import edu.sfedu.brsback.services.SecurityService;
import edu.sfedu.brsback.services.VisitService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.time.LocalDate;

@Service
@RequiredArgsConstructor
public class VisitServiceImpl implements VisitService {
    private final ModuleRightService moduleRightService;
    private final ModuleService moduleService;
    private final SecurityService securityService;
    private final VisitRepository visitRepository;
    private final RecordBookRepository recordBookRepository;
    private final VisitCreatedPublisher visitCreatedPublisher;
    @Override
    public void setVisit(LocalDate localDate, Module module, RecordBook recordBook) {
        checkAccess(module);

        Visit visit = new Visit(localDate, module, recordBook);

        visitCreatedPublisher.publish(visit, securityService.getAuthUser());
        visitRepository.save(visit);
    }

    @Override
    public void setVisit(VisitSetDto visitSetDto) {
        var module = moduleService.getModuleById(visitSetDto.getModuleId());
        checkAccess(module);

        Visit visit = new Visit();
        if (visitSetDto.getId() != null){
            visit = visitRepository.findById(visitSetDto.getId()).orElse(new Visit());
        }

        visit.setDate(visitSetDto.getDate());
        visit.setRecordBook(
                recordBookRepository.findById(visitSetDto.getRecordBookId()).orElseThrow(ModelNotFoundException::new)
        );
        visit.setModule(module);

        visitCreatedPublisher.publish(visit, securityService.getAuthUser());
        visitRepository.save(visit);
    }

    @Override
    public void setVisit(Visit visit) {
        checkAccess(visit.getModule());

        visitCreatedPublisher.publish(visit, securityService.getAuthUser());
        visitRepository.save(visit);
    }

    private void checkAccess(Module module) {
        var right = moduleRightService.getRightOnModule(
                module,
                securityService.getAuthUser()
        );

        if (right.isRScore()){
            throw new AccessDeniedException();
        }
    }
}
